@@ -11,11 +11,11 @@ from dagster.core.launcher.base import (
 )
 from dagster.core.storage.pipeline_run import PipelineRun
 from dagster.core.storage.tags import DOCKER_IMAGE_TAG
+from dagster.core.utils import parse_env_var
 from dagster.grpc.types import ExecuteRunArgs, ResumeRunArgs
 from dagster.serdes import ConfigurableClass
 
 from .container_context import DockerContainerContext
-from .utils import parse_env_var
 
 DOCKER_CONTAINER_ID_TAG = "docker/container_id"
 
@@ -140,7 +140,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
 
     def launch_run(self, context: LaunchRunContext) -> None:
         run = context.pipeline_run
-        pipeline_code_origin = context.pipeline_code_origin
+        pipeline_code_origin = check.not_none(context.pipeline_code_origin)
         docker_image = self._get_docker_image(pipeline_code_origin)
 
         command = ExecuteRunArgs(
@@ -157,7 +157,7 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
 
     def resume_run(self, context: ResumeRunContext) -> None:
         run = context.pipeline_run
-        pipeline_code_origin = context.pipeline_code_origin
+        pipeline_code_origin = check.not_none(context.pipeline_code_origin)
         docker_image = self._get_docker_image(pipeline_code_origin)
 
         command = ResumeRunArgs(
@@ -184,10 +184,6 @@ class DockerRunLauncher(RunLauncher, ConfigurableClass):
         except Exception:
             return None
 
-    def can_terminate(self, run_id):
-        run = self._instance.get_run_by_id(run_id)
-        return self._get_container(run) != None
-
     def terminate(self, run_id):
         run = self._instance.get_run_by_id(run_id)
         container = self._get_container(run)
