@@ -1,6 +1,3 @@
-import os
-from typing import Tuple
-
 from docker_image import reference
 
 from dagster import Field, StringSource
@@ -57,11 +54,3 @@ def validate_docker_image(docker_image):
                 docker_image=docker_image
             )
         ) from e
-
-
-def parse_env_var(env_var_str: str) -> Tuple[str, str]:
-    if "=" in env_var_str:
-        split = env_var_str.split("=")
-        return (split[0], split[1])
-    else:
-        return (env_var_str, os.getenv(env_var_str))
